'use strict';

/**
 * @name taskify
 *
 * @description
 * Lorem ipsum dolor sit amet
 *
 * @requires fs
 * @requires path
 * @requires gulp
 * @requires ./createTask
 *
 * @param {string} tasksDirPath
 * Lorem ipsum dolor sit amet
 *
 * @returns {array} tasks
 * Lorem ipsum dolor sit amet
 */
module.exports = function taskify (tasksDirPath) {

    //
    if (!tasksDirPath || typeof tasksDirPath !== 'string') {
        throw new Error();
    }

    var
        //
        fs   = require('fs'),
        path = require('path'),

        //
        gulp = require('gulp'),

        //
        createTask = require('./createTask.js'),

        // Chop the name off
        args = Array.prototype.slice.call(arguments, 1),

        //
        origDeps = [],

        //
        tasks;

    //
    try {
        tasksDirPath = path.normalize(tasksDirPath);
    } catch (err) {
        throw err;
    }

    //
    try {
        tasks = fs.lstatSync(tasksDirPath);
    } catch (err) {
        throw err;
    }

    //
    if (tasks.isDirectory()) {

        //
        try {
            tasks = fs.readdirSync(tasksDirPath);
        } catch (err) {
            throw err;
        }

        //
        if (!tasks.length) {
            throw new Error();
        }

    //
    } else if (tasks.isFile()) {
        tasks = [tasksDirPath];

    //
    } else {
        throw new Error();
    }

    //
    args.unshift(gulp);

    //
    tasks = tasks.map(function mapTasksFn (taskPath) {

        var
            //
            taskObj;

        //
        try {
            taskPath = path.join(tasksDirPath, taskPath);
        } catch (err) {
            throw err;
        }

        //
        try {
            taskObj = createTask(taskPath, args);
        } catch (err) {
            throw err;
        }

        // Shallow copy
        origDeps = taskObj.deps.slice();

        //
        taskObj.subtasks.forEach(function (subtaskObj) {

            //
            if (subtaskObj.auto) {

                // Don't pollute
                origDeps.forEach(function (dep) {

                    //
                    if (subtaskObj.deps.indexOf(dep) === -1) {
                        subtaskObj.deps.push(dep);
                    }

                });

                //
                taskObj.deps.push(subtaskObj.name);

            }

            //
            subtaskObj.alias.forEach(function (aliasName) {
                gulp.task(aliasName, subtaskObj.deps, subtaskObj.exports);
            });

            //
            gulp.task(subtaskObj.name, subtaskObj.deps, subtaskObj.exports);

        });

        //
        taskObj.alias.forEach(function (aliasName) {
            gulp.task(aliasName, taskObj.deps, taskObj.exports);
        });

        //
        gulp.task(taskObj.name, taskObj.deps, taskObj.exports);

        //
        return taskObj;

    });

    //
    return tasks;

};
